@@ -1,8 +1,7 @@
 use crate::chain::transaction::{DataInput, Input, UnsignedInput};
-use crate::chain::{
-    ergo_box::{ErgoBox, ErgoBoxCandidate},
-    transaction::TxId,
-};
+use ergotree_ir::chain::ergo_box::ErgoBox;
+use ergotree_ir::chain::ergo_box::ErgoBoxCandidate;
+use ergotree_ir::chain::tx_id::TxId;
 use serde::{Deserialize, Serialize};
 
 #[derive(Serialize, Deserialize, PartialEq, Debug, Clone)]
@@ -44,6 +43,8 @@ mod tests {
 
     proptest! {
 
+        #![proptest_config(ProptestConfig::with_cases(64))]
+
         #[test]
         fn tx_roundtrip(t in any::<Transaction>()) {
             let j = serde_json::to_string(&t)?;
