@@ -4,62 +4,38 @@ mod data_input;
 pub mod input;
 pub mod unsigned;
 
+use bounded_vec::BoundedVec;
+use ergotree_ir::chain::digest32::blake2b256_hash;
+use ergotree_ir::chain::ergo_box::ErgoBox;
+use ergotree_ir::chain::ergo_box::ErgoBoxCandidate;
+use ergotree_ir::chain::token::TokenId;
+pub use ergotree_ir::chain::tx_id::TxId;
+use thiserror::Error;
+
 pub use data_input::*;
+use ergotree_interpreter::sigma_protocol::prover::ProofBytes;
 use ergotree_ir::serialization::sigma_byte_reader::SigmaByteRead;
 use ergotree_ir::serialization::sigma_byte_writer::SigmaByteWrite;
-use ergotree_ir::serialization::SerializationError;
+use ergotree_ir::serialization::SigmaParsingError;
 use ergotree_ir::serialization::SigmaSerializable;
+use ergotree_ir::serialization::SigmaSerializationError;
+use ergotree_ir::serialization::SigmaSerializeResult;
 pub use input::*;
 
+use self::unsigned::UnsignedTransaction;
+
 #[cfg(feature = "json")]
 use super::json;
-use super::{
-    digest32::{blake2b256_hash, Digest32},
-    ergo_box::ErgoBox,
-    ergo_box::ErgoBoxCandidate,
-    token::TokenId,
-};
 use indexmap::IndexSet;
-#[cfg(test)]
-use proptest_derive::Arbitrary;
 #[cfg(feature = "json")]
 use serde::{Deserialize, Serialize};
 
 use std::convert::TryFrom;
-use std::io;
+use std::convert::TryInto;
 use std::iter::FromIterator;
-#[cfg(feature = "json")]
-use thiserror::Error;
 
-/// Transaction id (ModifierId in sigmastate)
-#[derive(PartialEq, Eq, Hash, Debug, Clone)]
-#[cfg_attr(test, derive(Arbitrary))]
-#[cfg_attr(feature = "json", derive(Serialize, Deserialize))]
-pub struct TxId(pub Digest32);
-
-impl TxId {
-    /// All zeros
-    pub fn zero() -> TxId {
-        TxId(Digest32::zero())
-    }
-}
-
-impl SigmaSerializable for TxId {
-    fn sigma_serialize<W: SigmaByteWrite>(&self, w: &mut W) -> Result<(), io::Error> {
-        self.0.sigma_serialize(w)?;
-        Ok(())
-    }
-    fn sigma_parse<R: SigmaByteRead>(r: &mut R) -> Result<Self, SerializationError> {
-        Ok(Self(Digest32::sigma_parse(r)?))
-    }
-}
-
-#[cfg(feature = "json")]
-impl From<TxId> for String {
-    fn from(v: TxId) -> Self {
-        v.0.into()
-    }
-}
+/// BouncedVec type for Tx inputs and output_candidates
+pub type TxIoVec<T> = BoundedVec<T, 1, { u16::MAX as usize }>;
 
 /**
  * ErgoTransaction is an atomic state transition operation. It destroys Boxes from the state
@@ -83,14 +59,19 @@ impl From<TxId> for String {
 pub struct Transaction {
     tx_id: TxId,
     /// inputs, that will be spent by this transaction.
-    pub inputs: Vec<Input>,
+    pub inputs: TxIoVec<Input>,
     /// inputs, that are not going to be spent by transaction, but will be reachable from inputs
     /// scripts. `dataInputs` scripts will not be executed, thus their scripts costs are not
     /// included in transaction cost and they do not contain spending proofs.
-    pub data_inputs: Vec<DataInput>,
-    /// box candidates to be created by this transaction. Differ from ordinary ones in that
+    pub data_inputs: Option<TxIoVec<DataInput>>,
+
+    /// box candidates to be created by this transaction. Differ from [`Self::outputs`] in that
     /// they do not include transaction id and index
-    pub output_candidates: Vec<ErgoBoxCandidate>,
+    pub output_candidates: TxIoVec<ErgoBoxCandidate>,
+
+    /// Boxes to be created by this transaction. Differ from [`Self::output_candidates`] in that
+    /// they include transaction id and index
+    pub outputs: Vec<ErgoBox>,
 }
 
 impl Transaction {
@@ -99,41 +80,65 @@ impl Transaction {
 
     /// Creates new transaction
     pub fn new(
-        inputs: Vec<Input>,
-        data_inputs: Vec<DataInput>,
-        output_candidates: Vec<ErgoBoxCandidate>,
-    ) -> Transaction {
+        inputs: TxIoVec<Input>,
+        data_inputs: Option<TxIoVec<DataInput>>,
+        output_candidates: TxIoVec<ErgoBoxCandidate>,
+    ) -> Result<Transaction, SigmaSerializationError> {
         let tx_to_sign = Transaction {
             tx_id: TxId::zero(),
             inputs,
             data_inputs,
-            output_candidates,
+            output_candidates: output_candidates.clone(),
+            outputs: vec![],
         };
-        let tx_id = tx_to_sign.calc_tx_id();
-        Transaction {
-            tx_id,
-            ..tx_to_sign
-        }
-    }
-
-    /// Returns ErgoBox's created from ErgoBoxCandidate's with tx id and indices
-    pub fn outputs(&self) -> Vec<ErgoBox> {
-        assert!(self.output_candidates.len() < u16::MAX as usize);
-        self.output_candidates
+        let tx_id = tx_to_sign.calc_tx_id()?;
+        let outputs = output_candidates
             .iter()
             .enumerate()
-            .map(|(idx, bc)| ErgoBox::from_box_candidate(bc, self.tx_id.clone(), idx as u16))
-            .collect()
+            .map(|(idx, bc)| ErgoBox::from_box_candidate(bc, tx_id.clone(), idx as u16))
+            .collect::<Result<Vec<ErgoBox>, SigmaSerializationError>>()?;
+        Ok(Transaction {
+            tx_id,
+            outputs,
+            ..tx_to_sign
+        })
     }
 
-    fn calc_tx_id(&self) -> TxId {
-        let bytes = self.bytes_to_sign();
-        TxId(blake2b256_hash(&bytes))
+    /// Create Transaction from UnsignedTransaction and an array of proofs in the same order as
+    /// UnsignedTransaction.inputs
+    pub fn from_unsigned_tx(
+        unsigned_tx: UnsignedTransaction,
+        proofs: Vec<ProofBytes>,
+    ) -> Result<Self, TransactionError> {
+        let inputs = unsigned_tx
+            .inputs
+            .enumerated()
+            .try_mapped(|(index, unsigned_input)| {
+                proofs
+                    .get(index)
+                    .map(|proof| Input::from_unsigned_input(unsigned_input, proof.clone()))
+                    .ok_or_else(|| {
+                        TransactionError::InvalidArgument(format!(
+                            "no proof for input index: {}",
+                            index
+                        ))
+                    })
+            })?;
+        Ok(Transaction::new(
+            inputs,
+            unsigned_tx.data_inputs,
+            unsigned_tx.output_candidates,
+        )?)
+    }
+
+    fn calc_tx_id(&self) -> Result<TxId, SigmaSerializationError> {
+        let bytes = self.bytes_to_sign()?;
+        Ok(TxId(blake2b256_hash(&bytes)))
     }
 
     /// Serialized tx with empty proofs
-    pub fn bytes_to_sign(&self) -> Vec<u8> {
-        let empty_proof_inputs = self.inputs.iter().map(|i| i.input_to_sign()).collect();
+    pub fn bytes_to_sign(&self) -> Result<Vec<u8>, SigmaSerializationError> {
+        let empty_proof_inputs = self.inputs.mapped_ref(|i| i.input_to_sign());
         let tx_to_sign = Transaction {
             inputs: empty_proof_inputs,
             ..(*self).clone()
@@ -147,39 +152,53 @@ impl Transaction {
     }
 }
 
+/// Returns distinct token ids from all given ErgoBoxCandidate's
+pub fn distinct_token_ids<I>(output_candidates: I) -> IndexSet<TokenId>
+where
+    I: IntoIterator<Item = ErgoBoxCandidate>,
+{
+    let token_ids: Vec<TokenId> = output_candidates
+        .into_iter()
+        .flat_map(|b| {
+            b.tokens
+                .iter()
+                .map(|t| t.token_id.clone())
+                .collect::<Vec<TokenId>>()
+        })
+        .collect();
+    IndexSet::<_>::from_iter(token_ids)
+}
+
 impl SigmaSerializable for Transaction {
-    fn sigma_serialize<W: SigmaByteWrite>(&self, w: &mut W) -> Result<(), io::Error> {
+    fn sigma_serialize<W: SigmaByteWrite>(&self, w: &mut W) -> SigmaSerializeResult {
         // reference implementation - https://github.com/ScorexFoundation/sigmastate-interpreter/blob/9b20cb110effd1987ff76699d637174a4b2fb441/sigmastate/src/main/scala/org/ergoplatform/ErgoLikeTransaction.scala#L112-L112
-        w.put_usize_as_u16(self.inputs.len())?;
+        w.put_usize_as_u16_unwrapped(self.inputs.len())?;
         self.inputs.iter().try_for_each(|i| i.sigma_serialize(w))?;
-        w.put_usize_as_u16(self.data_inputs.len())?;
-        self.data_inputs
-            .iter()
-            .try_for_each(|i| i.sigma_serialize(w))?;
+        if let Some(data_inputs) = &self.data_inputs {
+            w.put_usize_as_u16_unwrapped(data_inputs.len())?;
+            data_inputs.iter().try_for_each(|i| i.sigma_serialize(w))?;
+        } else {
+            w.put_u16(0)?;
+        }
 
         // Serialize distinct ids of tokens in transaction outputs.
         // This optimization is crucial to allow up to MaxTokens (== 255) in a box.
         // Without it total size of all token ids 255 * 32 = 8160, way beyond MaxBoxSize (== 4K)
-        let token_ids: Vec<TokenId> = self
-            .output_candidates
-            .iter()
-            .flat_map(|b| b.tokens.iter().map(|t| t.token_id.clone()))
-            .collect();
-        let distinct_token_ids: IndexSet<TokenId> = IndexSet::from_iter(token_ids);
+        let distinct_token_ids = distinct_token_ids(self.output_candidates.clone());
         w.put_u32(u32::try_from(distinct_token_ids.len()).unwrap())?;
         distinct_token_ids
             .iter()
             .try_for_each(|t_id| t_id.sigma_serialize(w))?;
 
         // serialize outputs
-        w.put_usize_as_u16(self.output_candidates.len())?;
+        w.put_usize_as_u16_unwrapped(self.output_candidates.len())?;
         self.output_candidates.iter().try_for_each(|o| {
             ErgoBoxCandidate::serialize_body_with_indexed_digests(o, Some(&distinct_token_ids), w)
         })?;
         Ok(())
     }
 
-    fn sigma_parse<R: SigmaByteRead>(r: &mut R) -> Result<Self, SerializationError> {
+    fn sigma_parse<R: SigmaByteRead>(r: &mut R) -> Result<Self, SigmaParsingError> {
         // reference implementation - https://github.com/ScorexFoundation/sigmastate-interpreter/blob/9b20cb110effd1987ff76699d637174a4b2fb441/sigmastate/src/main/scala/org/ergoplatform/ErgoLikeTransaction.scala#L146-L146
 
         // parse transaction inputs
@@ -199,7 +218,7 @@ impl SigmaSerializable for Transaction {
         // parse distinct ids of tokens in transaction outputs
         let tokens_count = r.get_u32()?;
         if tokens_count as usize > Transaction::MAX_OUTPUTS_COUNT * ErgoBox::MAX_TOKENS_COUNT {
-            return Err(SerializationError::ValueOutOfBounds(
+            return Err(SigmaParsingError::ValueOutOfBounds(
                 "too many tokens in transaction".to_string(),
             ));
         }
@@ -218,18 +237,36 @@ impl SigmaSerializable for Transaction {
             )?)
         }
 
-        Ok(Transaction::new(inputs, data_inputs, outputs))
+        Ok(Transaction::new(
+            inputs.try_into()?,
+            data_inputs.try_into().ok(),
+            outputs.try_into()?,
+        )?)
     }
 }
 
+/// Error when working with Transaction
+#[derive(Error, Eq, PartialEq, Debug, Clone)]
+pub enum TransactionError {
+    /// Serialization error
+    #[error("Tx serialization error: {0}")]
+    SigmaSerializationError(#[from] SigmaSerializationError),
+    /// Invalid argument on tx construction
+    #[error("Tx innvalid argument: {0}")]
+    InvalidArgument(String),
+}
+
 #[cfg(feature = "json")]
 impl From<Transaction> for json::transaction::TransactionJson {
     fn from(v: Transaction) -> Self {
         json::transaction::TransactionJson {
             tx_id: v.tx_id.clone(),
-            inputs: v.inputs.clone(),
-            data_inputs: v.data_inputs.clone(),
-            outputs: v.outputs(),
+            inputs: v.inputs.as_vec().clone(),
+            data_inputs: v
+                .data_inputs
+                .map(|di| di.as_vec().clone())
+                .unwrap_or_default(),
+            outputs: v.outputs,
         }
     }
 }
@@ -241,24 +278,37 @@ pub enum TransactionFromJsonError {
     /// Tx id parsed from JSON differs from calculated from serialized bytes
     #[error("Tx id parsed from JSON differs from calculated from serialized bytes")]
     InvalidTxId,
+    /// Serialization failed (id calculation)
+    #[error("Serialization failed (id calculation)")]
+    SerializationError,
+    /// Invalid tx input count
+    #[error("Invalid Tx inputs: {0:?}")]
+    InvalidInputsCount(bounded_vec::BoundedVecOutOfBounds),
+    /// Invalid tx output_candidates count
+    #[error("Invalid Tx output_candidates: {0:?}")]
+    InvalidOutputCandidatesCount(bounded_vec::BoundedVecOutOfBounds),
+    /// Invalid tx data inputs count
+    #[error("Invalid Tx data inputs: {0:?}")]
+    InvalidDataInputsCount(bounded_vec::BoundedVecOutOfBounds),
 }
 
 #[cfg(feature = "json")]
 impl TryFrom<json::transaction::TransactionJson> for Transaction {
     type Error = TransactionFromJsonError;
     fn try_from(tx_json: json::transaction::TransactionJson) -> Result<Self, Self::Error> {
-        let output_candidates = tx_json.outputs.iter().map(|o| o.clone().into()).collect();
-        let tx_to_sign = Transaction {
-            tx_id: TxId::zero(),
-            inputs: tx_json.inputs,
-            data_inputs: tx_json.data_inputs,
-            output_candidates,
-        };
-        let tx_id = tx_to_sign.calc_tx_id();
-        let tx = Transaction {
-            tx_id,
-            ..tx_to_sign
-        };
+        let output_candidates: Vec<ErgoBoxCandidate> =
+            tx_json.outputs.iter().map(|o| o.clone().into()).collect();
+        let tx = Transaction::new(
+            tx_json
+                .inputs
+                .try_into()
+                .map_err(TransactionFromJsonError::InvalidInputsCount)?,
+            tx_json.data_inputs.try_into().ok(),
+            output_candidates
+                .try_into()
+                .map_err(TransactionFromJsonError::InvalidOutputCandidatesCount)?,
+        )
+        .map_err(|_| TransactionFromJsonError::SerializationError)?;
         if tx.tx_id == tx_json.tx_id {
             Ok(tx)
         } else {
@@ -270,6 +320,8 @@ impl TryFrom<json::transaction::TransactionJson> for Transaction {
 #[cfg(test)]
 pub mod tests {
 
+    use std::convert::TryInto;
+
     use super::*;
 
     use ergotree_ir::serialization::sigma_serialize_roundtrip;
@@ -285,7 +337,14 @@ pub mod tests {
                 vec(any::<DataInput>(), 0..10),
                 vec(any::<ErgoBoxCandidate>(), 1..10),
             )
-                .prop_map(|(inputs, data_inputs, outputs)| Self::new(inputs, data_inputs, outputs))
+                .prop_map(|(inputs, data_inputs, outputs)| {
+                    Self::new(
+                        inputs.try_into().unwrap(),
+                        data_inputs.try_into().ok(),
+                        outputs.try_into().unwrap(),
+                    )
+                    .unwrap()
+                })
                 .boxed()
         }
         type Strategy = BoxedStrategy<Self>;
@@ -293,6 +352,8 @@ pub mod tests {
 
     proptest! {
 
+        #![proptest_config(ProptestConfig::with_cases(64))]
+
         #[test]
         fn tx_ser_roundtrip(v in any::<Transaction>()) {
             prop_assert_eq![sigma_serialize_roundtrip(&v), v];
