@@ -2,15 +2,20 @@
 
 pub mod prover_result;
 
-use std::io;
-
-use crate::chain::ergo_box::{BoxId, ErgoBoxId};
 use ergotree_interpreter::sigma_protocol::prover::ContextExtension;
 use ergotree_interpreter::sigma_protocol::prover::ProofBytes;
+use ergotree_ir::chain::ergo_box::BoxId;
 use ergotree_ir::serialization::sigma_byte_reader::SigmaByteRead;
 use ergotree_ir::serialization::sigma_byte_writer::SigmaByteWrite;
-use ergotree_ir::serialization::SerializationError;
+use ergotree_ir::serialization::SigmaParsingError;
 use ergotree_ir::serialization::SigmaSerializable;
+use ergotree_ir::serialization::SigmaSerializeResult;
+
+#[cfg(feature = "json")]
+use crate::chain::json::context_extension::ContextExtensionSerde;
+use crate::wallet::box_selector::ErgoBoxId;
+#[cfg(feature = "json")]
+use serde::ser::SerializeStruct;
 #[cfg(feature = "json")]
 use serde::{Deserialize, Serialize};
 
@@ -19,7 +24,7 @@ use self::prover_result::ProverResult;
 /// Unsigned (without proofs) transaction input
 #[derive(PartialEq, Debug, Clone)]
 #[cfg_attr(test, derive(proptest_derive::Arbitrary))]
-#[cfg_attr(feature = "json", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "json", derive(Deserialize))]
 pub struct UnsignedInput {
     /// id of the box to spent
     #[cfg_attr(feature = "json", serde(rename = "boxId"))]
@@ -33,6 +38,22 @@ pub struct UnsignedInput {
     pub extension: ContextExtension,
 }
 
+#[cfg(feature = "json")]
+impl Serialize for UnsignedInput {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        let mut s = serializer.serialize_struct("UnsignedInput", 2)?;
+        s.serialize_field("boxId", &self.box_id)?;
+        s.serialize_field(
+            "extension",
+            &ContextExtensionSerde::from(self.extension.clone()),
+        )?;
+        s.end()
+    }
+}
+
 impl UnsignedInput {
     /// Create new with empty ContextExtension
     pub fn new(box_id: BoxId, extension: ContextExtension) -> Self {
@@ -52,7 +73,7 @@ impl<T: ErgoBoxId> From<T> for UnsignedInput {
 #[cfg_attr(feature = "json", derive(Serialize, Deserialize))]
 pub struct Input {
     /// id of the box to spent
-    #[cfg_attr(feature = "json", serde(rename = "boxId"))]
+    #[cfg_attr(feature = "json", serde(rename = "boxId", alias = "id"))]
     pub box_id: BoxId,
     /// proof of spending correctness
     #[cfg_attr(feature = "json", serde(rename = "spendingProof",))]
@@ -68,6 +89,17 @@ impl Input {
         }
     }
 
+    /// Create Input from UnsignedInput and a proof
+    pub fn from_unsigned_input(unsigned_input: UnsignedInput, proof_bytes: ProofBytes) -> Self {
+        Self::new(
+            unsigned_input.box_id,
+            ProverResult {
+                proof: proof_bytes,
+                extension: unsigned_input.extension,
+            },
+        )
+    }
+
     /// input with an empty proof
     pub fn input_to_sign(&self) -> Input {
         Input {
@@ -81,12 +113,12 @@ impl Input {
 }
 
 impl SigmaSerializable for Input {
-    fn sigma_serialize<W: SigmaByteWrite>(&self, w: &mut W) -> Result<(), io::Error> {
+    fn sigma_serialize<W: SigmaByteWrite>(&self, w: &mut W) -> SigmaSerializeResult {
         self.box_id.sigma_serialize(w)?;
         self.spending_proof.sigma_serialize(w)?;
         Ok(())
     }
-    fn sigma_parse<R: SigmaByteRead>(r: &mut R) -> Result<Self, SerializationError> {
+    fn sigma_parse<R: SigmaByteRead>(r: &mut R) -> Result<Self, SigmaParsingError> {
         let box_id = BoxId::sigma_parse(r)?;
         let spending_proof = ProverResult::sigma_parse(r)?;
         Ok(Input {
