@@ -1,30 +1,35 @@
 //! Builder for an UnsignedTransaction
 
+use ergotree_ir::chain::ergo_box::box_value::BoxValueError;
 use std::collections::HashSet;
+use std::convert::TryInto;
 
+use bounded_vec::BoundedVecOutOfBounds;
 use ergotree_interpreter::sigma_protocol;
 use ergotree_interpreter::sigma_protocol::prover::ProofBytes;
-use ergotree_ir::address::{Address, AddressEncoder, NetworkPrefix};
-use ergotree_ir::serialization::SerializationError;
-use ergotree_ir::serialization::SigmaSerializable;
+use ergotree_ir::chain::address::Address;
+use ergotree_ir::chain::address::AddressEncoder;
+use ergotree_ir::chain::address::NetworkPrefix;
+use ergotree_ir::chain::ergo_box::box_value::BoxValue;
+use ergotree_ir::chain::ergo_box::BoxId;
+use ergotree_ir::chain::ergo_box::ErgoBoxCandidate;
+use ergotree_ir::chain::token::Token;
+use ergotree_ir::chain::token::TokenId;
+use ergotree_ir::serialization::{SigmaParsingError, SigmaSerializable, SigmaSerializationError};
 use thiserror::Error;
 
 use crate::chain::contract::Contract;
 use crate::chain::ergo_box::box_builder::{ErgoBoxCandidateBuilder, ErgoBoxCandidateBuilderError};
-use crate::chain::ergo_box::{sum_tokens_from_boxes, sum_value, BoxId, BoxValue, BoxValueError};
-use crate::chain::token::{Token, TokenId};
+use crate::chain::transaction::unsigned::UnsignedTransaction;
 use crate::chain::transaction::{DataInput, Input, Transaction, UnsignedInput};
-use crate::chain::{
-    ergo_box::ErgoBoxAssets, ergo_box::ErgoBoxCandidate, ergo_box::ErgoBoxId,
-    transaction::unsigned::UnsignedTransaction,
-};
 use crate::constants::MINERS_FEE_MAINNET_ADDRESS;
 
+use super::box_selector::sum_tokens_from_boxes;
+use super::box_selector::sum_value;
+use super::box_selector::ErgoBoxAssets;
+use super::box_selector::ErgoBoxId;
 use super::box_selector::{BoxSelection, BoxSelectorError};
 
-/// Suggested transaction fee (1100000 nanoERGs, semi-default value used across wallets and dApps as of Oct 2020)
-pub const SUGGESTED_TX_FEE: BoxValue = BoxValue(1100000u64);
-
 /// Unsigned transaction builder
 #[derive(Clone)]
 pub struct TxBuilder<S: ErgoBoxAssets> {
@@ -108,24 +113,20 @@ impl<S: ErgoBoxAssets + ErgoBoxId + Clone> TxBuilder<S> {
     /// Estimated serialized transaction size in bytes after signing (assuming P2PK box spending)
     pub fn estimate_tx_size_bytes(&self) -> Result<usize, TxBuilderError> {
         let tx = self.build_tx()?;
-        let inputs = tx
-            .inputs
-            .iter()
-            .map(|ui| {
-                // mock proof of the size of ProveDlog's proof (P2PK box spending)
-                // as it's the most often used proof
-                let proof = ProofBytes::Some(vec![0u8, sigma_protocol::SOUNDNESS_BYTES as u8]);
-                Input::new(
-                    ui.box_id.clone(),
-                    crate::chain::transaction::input::prover_result::ProverResult {
-                        proof,
-                        extension: ui.extension.clone(),
-                    },
-                )
-            })
-            .collect();
-        let signed_tx_mock = Transaction::new(inputs, tx.data_inputs, tx.output_candidates);
-        Ok(signed_tx_mock.sigma_serialize_bytes().len())
+        let inputs = tx.inputs.mapped(|ui| {
+            // mock proof of the size of ProveDlog's proof (P2PK box spending)
+            // as it's the most often used proof
+            let proof = ProofBytes::Some(vec![0u8, sigma_protocol::SOUNDNESS_BYTES as u8]);
+            Input::new(
+                ui.box_id.clone(),
+                crate::chain::transaction::input::prover_result::ProverResult {
+                    proof,
+                    extension: ui.extension,
+                },
+            )
+        });
+        let signed_tx_mock = Transaction::new(inputs, tx.data_inputs, tx.output_candidates)?;
+        Ok(signed_tx_mock.sigma_serialize_bytes()?.len())
     }
 
     fn build_tx(&self) -> Result<UnsignedTransaction, TxBuilderError> {
@@ -227,10 +228,11 @@ impl<S: ErgoBoxAssets + ErgoBoxId + Clone> TxBuilder<S> {
                 .clone()
                 .into_iter()
                 .map(UnsignedInput::from)
-                .collect(),
-            self.data_inputs.clone(),
-            output_candidates,
-        ))
+                .collect::<Vec<UnsignedInput>>()
+                .try_into()?,
+            self.data_inputs.clone().try_into().ok(),
+            output_candidates.try_into()?,
+        )?)
     }
 
     /// Build the unsigned transaction
@@ -239,6 +241,12 @@ impl<S: ErgoBoxAssets + ErgoBoxId + Clone> TxBuilder<S> {
     }
 }
 
+/// Suggested transaction fee (1100000 nanoERGs, semi-default value used across wallets and dApps as of Oct 2020)
+#[allow(non_snake_case)]
+pub fn SUGGESTED_TX_FEE() -> BoxValue {
+    BoxValue::new(1100000u64).unwrap()
+}
+
 /// Create a box with miner's contract and a given value
 pub fn new_miner_fee_box(
     fee_amount: BoxValue,
@@ -261,9 +269,9 @@ pub enum TxBuilderError {
     /// Box value error
     #[error("Box value error")]
     BoxValueError(#[from] BoxValueError),
-    /// Serialization error
-    #[error("Serialization error")]
-    SerializationError(#[from] SerializationError),
+    /// Parsing error
+    #[error("Parsing error: {0}")]
+    ParsingError(#[from] SigmaParsingError),
     /// Invalid arguments
     #[error("Invalid arguments: {0}")]
     InvalidArgs(String),
@@ -276,6 +284,12 @@ pub enum TxBuilderError {
     /// Not enough coins
     #[error("Not enough coins({0} nanoERGs are missing)")]
     NotEnoughCoins(u64),
+    /// Tx serialization failed (id calculation)
+    #[error("Transaction serialization failed: {0}")]
+    SerializationError(#[from] SigmaSerializationError),
+    /// Invalid Tx input count
+    #[error("Invalid tx inputs count: {0}")]
+    InvalidInputsCount(#[from] BoundedVecOutOfBounds),
 }
 
 #[cfg(test)]
@@ -283,16 +297,17 @@ mod tests {
 
     use std::convert::TryInto;
 
+    use ergotree_ir::chain::ergo_box::box_value::checked_sum;
+    use ergotree_ir::chain::ergo_box::ErgoBox;
+    use ergotree_ir::chain::ergo_box::NonMandatoryRegisters;
+    use ergotree_ir::chain::token::arbitrary::ArbTokenIdParam;
+    use ergotree_ir::chain::token::TokenAmount;
+    use ergotree_ir::chain::tx_id::TxId;
     use ergotree_ir::ergo_tree::ErgoTree;
     use proptest::{collection::vec, prelude::*};
     use sigma_test_util::force_any_val;
     use sigma_test_util::force_any_val_with;
 
-    use crate::chain::{
-        ergo_box::{checked_sum, ErgoBox, NonMandatoryRegisters},
-        token::{tests::ArbTokenIdParam, Token, TokenAmount, TokenId},
-        transaction::TxId,
-    };
     use crate::wallet::box_selector::{BoxSelector, SimpleBoxSelector};
 
     use super::*;
@@ -363,7 +378,8 @@ mod tests {
             1,
             force_any_val::<TxId>(),
             0,
-        );
+        )
+        .unwrap();
         let inputs: Vec<ErgoBox> = vec![input_box];
         let tx_fee = BoxValue::SAFE_USER_MIN;
         let out_box_value = BoxValue::SAFE_USER_MIN;
@@ -404,7 +420,8 @@ mod tests {
             1,
             force_any_val::<TxId>(),
             0,
-        );
+        )
+        .unwrap();
         let token_pair = Token {
             token_id: TokenId::from(input_box.box_id()),
             amount: 1.try_into().unwrap(),
@@ -526,8 +543,9 @@ mod tests {
             1,
             force_any_val::<TxId>(),
             0,
-        );
-        let tx_fee = super::SUGGESTED_TX_FEE;
+        )
+        .unwrap();
+        let tx_fee = super::SUGGESTED_TX_FEE();
         let out_box_value = input.value.checked_sub(&tx_fee).unwrap();
         let box_builder =
             ErgoBoxCandidateBuilder::new(out_box_value, force_any_val::<ErgoTree>(), 0);
@@ -578,11 +596,10 @@ mod tests {
             let tx = tx_builder.build().unwrap();
             prop_assert!(outputs.into_iter().all(|i| tx.output_candidates.iter().any(|o| *o == i)),
                          "tx.output_candidates is missing some outputs");
-            let tx_all_inputs_vals: Vec<BoxValue> = tx.inputs.iter()
-                                                             .map(|i| inputs.iter()
-                                                                  .find(|ib| ib.box_id() == i.box_id).unwrap().value)
-                                                             .collect();
-            let tx_all_inputs_sum = checked_sum(tx_all_inputs_vals.into_iter()).unwrap();
+            let tx_all_inputs_vals = tx.inputs.iter()
+                .map(|i| inputs.iter()
+                    .find(|ib| ib.box_id() == i.box_id).unwrap().value);
+            let tx_all_inputs_sum = checked_sum(tx_all_inputs_vals).unwrap();
             let expected_change = tx_all_inputs_sum.checked_sub(&all_outputs).unwrap();
             prop_assert!(tx.output_candidates.iter().any(|b| {
                 b.value == expected_change && b.ergo_tree == change_address.script().unwrap()
@@ -590,7 +607,7 @@ mod tests {
             prop_assert!(tx.output_candidates.iter().any(|b| {
                 b.value == miners_fee
             }), "box with miner's fee {:?} is not found in outputs: {:?}", miners_fee, tx.output_candidates);
-            prop_assert_eq!(tx.data_inputs, data_inputs, "unexpected data inputs");
+            prop_assert_eq!(tx.data_inputs.map(|i| i.as_vec().clone()).unwrap_or_default(), data_inputs, "unexpected data inputs");
         }
     }
 }
